/**
 * Copyright (c) 2000-2011 Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.samplestruts.struts.action;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;

import java.io.OutputStream;

import java.util.Enumeration;

import javax.portlet.PortletSessionUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.PageContext;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.data.general.DatasetUtilities;
import org.jfree.data.general.PieDataset;

/**
 * @author Brian Wing Shun Chan
 */
public class ViewChartAction extends Action {

	@Override
	public ActionForward execute(
			ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response)
		throws Exception {

		try {
			if (_log.isInfoEnabled()) {
				_log.info("execute");
			}

			String portletName = "sample_struts_portlet";
			String attrName = "chart_name";

			// Application scoped session attributes can be fetched from the
			// servlet directly. Portlet scoped session attributes can be
			// fetched from Sun's PortletSessionUtil.

			HttpSession session = request.getSession();

			String chartName =
				(String)session.getAttribute(attrName);
				//(String)_getAttribute(request, attrName);

			// Chart

			String chartType = request.getParameter("chart_type");

			CategoryDataset dataset = _getDataset();

			String xName = "Soda";
			String yName = "Votes";

			JFreeChart chart = null;

			if (chartType.equals("area")) {
				chart = ChartFactory.createAreaChart(
					chartName, xName, yName, dataset,
					PlotOrientation.VERTICAL, true, false, false);
			}
			else if (chartType.equals("horizontal_bar")) {
				chart = ChartFactory.createBarChart(
					chartName, xName, yName, dataset,
					PlotOrientation.HORIZONTAL, true, false, false);
			}
			else if (chartType.equals("line")) {
				chart = ChartFactory.createLineChart(
					chartName, xName, yName, dataset,
					PlotOrientation.VERTICAL, true, false, false);
			}
			else if (chartType.equals("vertical_bar")) {
				chart = ChartFactory.createBarChart(
					chartName, xName, yName, dataset,
					PlotOrientation.VERTICAL, true, false, false);
			}
			else {
				PieDataset pieData =
					DatasetUtilities.createPieDatasetForRow(dataset, 0);

				chart = ChartFactory.createPieChart(
					chartName, pieData, true, false, false);
			}

			response.setContentType("image/jpeg");

			OutputStream out = response.getOutputStream();

			ChartUtilities.writeChartAsJPEG(out, chart, 400, 400);

			return mapping.findForward("/common/null.jsp");
		}
		catch (Exception e) {
			request.setAttribute(PageContext.EXCEPTION, e);

			return mapping.findForward("/common/error.jsp");
		}
	}

	private Object _getAttribute(HttpServletRequest request, String attrName) {

		// Go through all the session attributes and use Sun's
		// PortletSessionUtil to match the correct attribute name

		HttpSession session = request.getSession();

		Enumeration enu = session.getAttributeNames();

		while (enu.hasMoreElements()) {
			String encodedAttrName = (String)enu.nextElement();

			String decodedAttrName =
				PortletSessionUtil.decodeAttributeName(encodedAttrName);

			if (decodedAttrName.equals(attrName)) {
				return session.getAttribute(encodedAttrName);
			}
		}

		return null;
	}

	private CategoryDataset _getDataset() throws Exception {
		DefaultCategoryDataset dataset = new DefaultCategoryDataset();

		String seriesName = "";

		dataset.addValue(new Integer(9), seriesName, "Coke");
		dataset.addValue(new Integer(21), seriesName, "Pepsi");

		return dataset;
	}

	private static Log _log = LogFactoryUtil.getLog(ViewChartAction.class);

}